@@ -14,7 +14,7 @@ __author__ = (
     "Mark Harviston <mark.harviston@gmail.com>, "
     "Arve Knudsen <arve.knudsen@gmail.com>",
 )
-__version__ = "0.14.0"
+__version__ = "0.16.0"
 __url__ = "https://github.com/CabbageDevelopment/qasync"
 __license__ = "BSD"
 __all__ = ["QEventLoop", "QThreadExecutor", "asyncSlot", "asyncClose"]
@@ -298,6 +298,10 @@ class _QEventLoop:
     >>> asyncio.set_event_loop(loop)
     >>> with loop:
     ...     loop.run_until_complete(xplusy(2, 2))
+
+    If the event loop shall be used with an existing and already running QApplication
+    it must be specified in the constructor via already_running=True
+    In this case the user is responsible for loop cleanup with stop() and close()
     """
 
     def __init__(self, app=None, set_running_loop=True, already_running=False):
@@ -326,8 +330,17 @@ class _QEventLoop:
         if already_running:
             self.__is_running = True
 
+            # it must be ensured that all pre- and
+            # postprocessing for the eventloop is done
+            self._before_run_forever()
+            self.__app.aboutToQuit.connect(self._after_run_forever)
+
     def run_forever(self):
         """Run eventloop forever."""
+
+        if self.__is_running:
+            raise RuntimeError("Event loop already running")
+
         self.__is_running = True
         self._before_run_forever()
 
@@ -342,6 +355,10 @@ class _QEventLoop:
 
     def run_until_complete(self, future):
         """Run until Future is complete."""
+
+        if self.__is_running:
+            raise RuntimeError("Event loop already running")
+
         self._logger.debug("Running {} until complete".format(future))
         future = asyncio.ensure_future(future, loop=self)
 
@@ -718,11 +735,19 @@ def asyncClose(fn):
 def asyncSlot(*args):
     """Make a Qt async slot run on asyncio loop."""
 
+    def _error_handler(task):
+        try:
+            task.result()
+        except Exception:
+            sys.excepthook(*sys.exc_info())
+
     def outer_decorator(fn):
         @Slot(*args)
         @functools.wraps(fn)
         def wrapper(*args, **kwargs):
-            return asyncio.ensure_future(fn(*args, **kwargs))
+            task = asyncio.ensure_future(fn(*args, **kwargs))
+            task.add_done_callback(_error_handler)
+            return task
 
         return wrapper
 
